﻿using System;
using System.Drawing;
using System.Linq;
using System.Net;
using System.Windows.Forms;
using Microscan.SDK;
using MSDK = Microscan.SDK;

namespace SDK.NET_1._0_Example
{
	public partial class Form1 : Form
	{
		const string DefaultAddr = @"192.168.188.2";
		MSDK.ReaderDevice _reader;
		MSDK.ReaderCycleReportConnection _reportConnection;
		MSDK.ImageOptions _imageOptions;
		readonly MSDK.DeviceDiscovery _discovery = MSDK.Connector.Instance.Discovery;

		public Form1()
		{
			InitializeComponent();

			// fill combo boxes - reference documentation for all Microscan.SDK options

			// image fetch
			comboImages.Items.Add("None");
			comboImages.Items.Add("First Qualified");

			// image format
			comboFormat.Items.Add("PNG");
			comboFormat.Items.Add("JPG");

			// decimate
			comboDecimate.Items.Add("0");
			comboDecimate.Items.Add("1");
			comboDecimate.Items.Add("2");
			comboDecimate.Items.Add("3");

			// graphics
			comboGraphics.Items.Add("Off");
			comboGraphics.Items.Add("On");

			AcceptButton = buttonStart; //this allows the user to type in the ip address in and click enter to connect

			ipAddressCtrl.Text = DefaultAddr;

			UseReader(ipAddressCtrl.Text);

			//required if you want to be notified of new devices
			//you must StartScan() if you want the SDK to search for devices
			MSDK.DeviceDiscovery.NewDevice += DeviceDiscovery_NewDevice;
		}

		void ipAddressCtrl_KeyDown(object sender, KeyEventArgs e)
		{
			if (e.KeyCode == Keys.Return)
			{
				toggleStart();
				e.Handled = true;
			}
		}

		private void UseReader(string ipAddress)
		{
			// get the Connector
			var connector = MSDK.Connector.Instance;

			if (_reportConnection != null)
			{
				_reportConnection.Closed -= ReportConnection_Closed;
				_reportConnection.Opened -= ReportConnection_Opened;
				_reportConnection.NewReport -= ReportConnection_NewReport;
				_reportConnection.NewImageReport -= ReportConnection_NewImageReport;
			}
			_reportConnection = null;
			_reader = null;

			// use a reader device
			_reader = connector.UseReader(ipAddress);

			// configure the report connection
			_reportConnection = _reader.ReportConnection;
			_reportConnection.Throttle = 300; //in ms - prevent reports from coming in too fast

			// handle common events
			_reportConnection.Closed += ReportConnection_Closed;
			_reportConnection.Opened += ReportConnection_Opened;
			_reportConnection.NewReport += ReportConnection_NewReport;
			_reportConnection.NewImageReport += ReportConnection_NewImageReport;


			// set any image options
			_imageOptions = _reportConnection.ImageOptions;

			_imageOptions.ImageFetch = MSDK.IMAGE_FETCH.FIRST_QUALIFIED;
			_imageOptions.ImageFormat = MSDK.IMAGE_FORMAT.PNG;
			_imageOptions.Decimate = 0;
			_imageOptions.DrawGraphics = true;

			// update the form controls
			UpdateImageOptionControls();
			UpdateButtonStates();
		}

		private void Form1_FormClosing(object sender, FormClosingEventArgs e)
		{
			// IMPORTANT - Do these items when closing the application!
			// stop the connection!
			_reportConnection.Stop();
			_discovery.StopScan();

			// also DoEvents to flush any pending events before handlers are destroyed
			Application.DoEvents();
		}

		//**************************************************************************************************************
		// Start / Stop Button
		//**************************************************************************************************************
		private void buttonStart_Click(object sender, EventArgs e)
		{
			toggleStart();
		}
		
		private void toggleStart()
		{
			if (_reportConnection.IsOpen())
			{
				_reportConnection.Stop();
			}
			else
			{
				if (_reader.IpAddr != ipAddressCtrl.Text)
				{
					UseReader(ipAddressCtrl.Text);
				}
				_reportConnection.Start();
			}
			UpdateButtonStates();
		}
		

		//**************************************************************************************************************
		// WDK Events
		//**************************************************************************************************************
		private void ReportConnection_Opened(object o)
		{
			UpdateButtonStates();
		}

		private void ReportConnection_Closed(object o)
		{
			UpdateButtonStates();
		}

		private void ReportConnection_NewImageReport(MSDK.ImageReport imageReport)
		{
			imagePictureBox.Image = imageReport.Image;

			imagePanel.BackColor = imageReport.CycleReport.decodeStatus ? Color.Green : Color.Red;
			imagePanel.Padding = new Padding(2);
		}

		private void ReportConnection_NewReport(MSDK.ReaderCycleReport report)
		{
			tOutData.Text = report.outData;
			tReadTime.Text = String.Format("{0} ms", report.timingInfo.readTime);
		}

		//**************************************************************************************************************
		// Handle Combos, etc 
		//**************************************************************************************************************
		private void UpdateImageOptionControls()
		{
			if (null == _imageOptions)
			{
				return;
			}

			comboImages.SelectedIndex = comboImages.Items.IndexOf(_imageOptions.ImageFetch == MSDK.IMAGE_FETCH.NONE ? "None" : "First Qualified");
			comboDecimate.SelectedIndex = comboDecimate.Items.IndexOf(_imageOptions.Decimate.ToString());
			comboFormat.SelectedIndex = comboFormat.Items.IndexOf(_imageOptions.ImageFormat.ToString());
			comboGraphics.SelectedIndex = comboGraphics.Items.IndexOf(_imageOptions.DrawGraphics ? "On" : "Off");
		}

		private void UpdateButtonStates()
		{
			if (_reportConnection.IsOpen())
			{
				buttonStart.Text = "Stop";
				buttonStart.BackColor = Color.LightGreen;
				bttnGetLastReport.Enabled = true;
				bttnGetLastImage.Enabled = true;
				ipAddressCtrl.Enabled = false;
				buttonScan.Enabled = false;
				comboDevices.Enabled = false;
			}
			else
			{
				buttonStart.Text = "Start";
				buttonStart.BackColor = Color.Red;
				bttnGetLastReport.Enabled = false;
				bttnGetLastImage.Enabled = false;
				ipAddressCtrl.Enabled = true;
				buttonScan.Enabled = true;
				comboDevices.Enabled = true;
			}
		}

		//handle the user changes
		private void comboFormat_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (_imageOptions == null || comboFormat.SelectedIndex < 0) return;
			_imageOptions.ImageFormat = comboFormat.Items[comboFormat.SelectedIndex].ToString() == "PNG" ? MSDK.IMAGE_FORMAT.PNG : MSDK.IMAGE_FORMAT.JPG;

		}

		private void comboDecimate_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (_imageOptions == null || comboDecimate.SelectedIndex < 0) return;
			_imageOptions.Decimate = int.Parse(comboDecimate.Items[comboDecimate.SelectedIndex].ToString());
		}

		private void comboImages_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (_imageOptions == null || comboImages.SelectedIndex < 0) return;
			_imageOptions.ImageFetch = comboImages.Items[comboImages.SelectedIndex].ToString() == "None" ? MSDK.IMAGE_FETCH.NONE : MSDK.IMAGE_FETCH.FIRST_QUALIFIED;
		}

		private void comboGraphics_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (_imageOptions == null || comboGraphics.SelectedIndex < 0) return;
			_imageOptions.DrawGraphics = comboGraphics.Items[comboGraphics.SelectedIndex].ToString() == "On";
		}

		private void bttnGetLastReport_Click(object sender, EventArgs e)
		{
			if (!_reportConnection.IsOpen()) return;
			//get the last report
			var report = _reportConnection.LatestReport;
			if (report != null)
			{
				tOutData.Text = report.outData;
				tReadTime.Text = String.Format("{0} ms", report.timingInfo.readTime);
			}
		}

		private void bttnGetLastImage_Click(object sender, EventArgs e)
		{
			if (!_reportConnection.IsOpen() || (_reportConnection.LatestImage == null)) return;
			//get the last image - make sure to connect first
			imagePictureBox.Image = _reportConnection.LatestImage;

			imagePanel.BackColor = Color.Empty;
		}

		private void buttonScan_click(object sender, EventArgs e)
		{
			//search for devices (uses a UDP broadcast). Notification via the NewDevice event
			_discovery.StartScan();
		}

		//callback for the device discovery. Called when a device is found on the network.
		private void DeviceDiscovery_NewDevice(UdpDevice dev)
		{
			UpdateDevices();
		}

		//Update the device list combobox
		private void UpdateDevices()
		{
			var list = _discovery.DeviceList;
			foreach (var dev in list)
			{
				if (!comboDevices.Items.Contains(dev.ipAddr.ToString()))
				{
					comboDevices.Items.Add(dev.ipAddr.ToString());
				}
			}
		}

		//When the combobox selection changes then update the device info text box and ipAddressCtrl.
		private void comboDevices_SelectedIndexChanged(object sender, EventArgs e)
		{
			var comboBox = (ComboBox)sender;
			var ipAddr = comboBox.SelectedItem;
			var list = _discovery.DeviceList;
			
			UdpDevice dev = null;
			for (var i = 0; i < list.Length; i++)
			{
				if (ipAddr.Equals(list[i].ipAddr.ToString()))
				{
					dev = list[i];
					break;
				}
			}
			if (dev != null)
			{
				var info = dev.devInfo;
				tDeviceInfo.Text = string.Format("{0}\n{1}\n{2}", info.name, info.model, info.macAddr);
				IPAddress addr;
				if (IPAddress.TryParse(ipAddr.ToString(), out addr))
				{
					ipAddressCtrl.Text = addr.ToString();
				}
			}
		}
	}
}
